#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Moduł interfejsu wiersza poleceń (CLI) dla AI Environment Manager.

Zapewnia interfejs wiersza poleceń do zarządzania maszynami wirtualnymi,
kontenerami i innymi zasobami w środowisku AI Environment Manager.
"""

# Avoid direct imports here to prevent circular imports
# The main function will be imported directly by the entry point
